@@ -31,7 +31,7 @@ if USE_GUI:
     import pykat.gui.resources
     from pykat.gui.graphics import *
 
-id___ = 0
+id_____pykat_class = 0
 
 class BaseDetector(object) :
     """
@@ -45,9 +45,9 @@ class BaseDetector(object) :
         # This creates an instance specific class for the component
         # this enables us to add properties to instances rather than
         # all classes
-        global id___
-        id___ += 1
-        cnew_name = str("%s.%s_%i" % (cls.__module__, cls.__name__, id___))
+        global id_____pykat_class
+        id_____pykat_class += 1
+        cnew_name = str("%s.%s_%i" % (cls.__module__, cls.__name__, id_____pykat_class))
         
         cnew = type(cnew_name, (cls,), {})
         
@@ -65,6 +65,7 @@ class BaseDetector(object) :
         self._mask = {}
         self.__scale = []
         self.__removed = False
+        self.noplot = False
         
         self._alternate_beam = []
         self._nodes = []
@@ -119,6 +120,9 @@ class BaseDetector(object) :
             if rn != None:
                 self._nodes.append(kat.nodes.createNode(rn))
     
+    def _on_kat_remove(self):
+        self.__removed = True
+        
     def remove(self):
         if self.__removed:
             raise pkex.BasePyKatException("{0} has already been marked as removed".format(self.name))
@@ -133,7 +137,12 @@ class BaseDetector(object) :
         
     def getFinesseText(self):
         """ Base class for individual finesse optical components """    
-        raise NotImplementedError("This function is not implemented")
+        #raise NotImplementedError("This function is not implemented")
+        
+        if self.noplot:
+            return ["noplot " + self.name]
+        else:
+            return []
         
     def getQGraphicsItem(self):    
         if not USE_GUI:
@@ -190,7 +199,13 @@ class BaseDetector(object) :
                     rtn.append("scale {1} {0}".format(self.name, s))
             else:
                 raise pkex.BasePyKatException("Scale command should either be a list of scales or a single string.")
-                
+
+class Detector0(BaseDetector):
+    """
+    A detector that attaches to no nodes.
+    """
+    pass
+                        
 class Detector1(BaseDetector):
     """
     A detector that attaches to one node.
@@ -251,7 +266,7 @@ class beam(Detector1):
             raise pkex.BasePyKatException('Beam detector code "{0}" is not a valid FINESSE command'.format(text))
     
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.alternate_beam:
             alt = '*'
@@ -262,13 +277,93 @@ class beam(Detector1):
             rtn.append("beam {name} {node}{alt}".format(name=self.name, node=self.node.name, alt=alt))
         else:
             rtn.append("beam {name} {f} {node}{alt}".format(name=self.name, f=str(self.f.value), node=self.node.name, alt=alt))
-            
+        
         for p in self._params:
             rtn.extend(p.getFinesseText())
         
         return rtn
         
+class cp(Detector0):
+    
+    def __init__(self, name, cavity, direction, parameter):
+        BaseDetector.__init__(self, name, None)
         
+        self.cavity = str(cavity)
+        self.direction = direction
+        self.parameter = parameter
+
+    @property
+    def direction(self): return self.__direction
+    @direction.setter
+    def direction(self, value):
+        if value.lower() not in ["x", "y"]:
+            raise pkex.BasePyKatException('Cavity parameter detector direction must be x or y.')
+    
+        self.__direction = value
+        
+        
+    @property
+    def parameter(self): return self.__param
+    @parameter.setter
+    def parameter(self, value):
+        
+        params = ["w0","w","zr","z","r","q","finesse","m","stability","loss","length","fsr","fwhm","pole","gouy","fsep","A","B","C","D"]
+
+        if value not in params:
+            raise pkex.BasePyKatException('Cavity parameter detector direction must be one of: ' + ", ".join(params))
+    
+        self.__param = value
+
+
+    @staticmethod
+    def parseFinesseText(text): 
+        values = text.split()
+        
+        if len(values) == 4:
+            # For FINESSE version < 2.1
+            # Old format the name of the detector is a combination of the arguments
+            return cp(values[1] + "_" + values[2] + "_" + values[3], values[1], values[2], values[3])
+        elif len(values) == 5:
+            return cp(values[1], values[2], values[3], values[4])
+        else:
+            raise pkex.BasePyKatException('Cavity parameter detector code "{0}" is not a valid FINESSE command'.format(text))
+            
+    def getFinesseText(self) :
+        rtn = BaseDetector.getFinesseText(self)
+        
+        rtn.append("cp {name} {cavity} {direction} {parameter}".format(name=self.name,
+                                                                   cavity=str(self.cavity),
+                                                                   direction=self.direction,
+                                                                   parameter=self.parameter))
+        
+        return rtn
+        
+class xd(Detector0):
+    
+    def __init__(self, name, component, motion):
+        BaseDetector.__init__(self, name, None)
+        
+        self.component = component
+        self.motion = motion
+
+    @staticmethod
+    def parseFinesseText(text): 
+        values = text.split()
+        
+        if len(values) == 4:
+            return xd(values[1], values[2], values[3])
+        else:
+            raise pkex.BasePyKatException('Motion detector code "{0}" is not a valid FINESSE command'.format(text))
+            
+    def getFinesseText(self) :
+        rtn = BaseDetector.getFinesseText(self)
+        
+        rtn.append("xd {name} {component} {motion}".format(name=self.name,
+                                                           component=self.component,
+                                                           motion=self.motion))
+        
+        return rtn
+              
         
 class ad(Detector1):
     
@@ -307,7 +402,7 @@ class ad(Detector1):
             raise pkex.BasePyKatException('Amplitude detector code "{0}" is not a valid FINESSE command'.format(text))
             
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.alternate_beam:
             alt = '*'
@@ -361,7 +456,7 @@ class gouy(Detector1):
             raise pkex.BasePyKatException('Gouy detector code "{0}" is not a valid FINESSE command'.format(text))
             
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
 
         rtn.append("gouy {name} {dir} {spaces}".format(name=self.name, dir=str(self.direction), spaces=" ".join(self.spaces)))
         
@@ -373,7 +468,7 @@ class gouy(Detector1):
 
 
 class bp(Detector1):
-    acceptedParameters = ['w', 'w0', 'z', 'zr', 'g', 'r', 'q']
+    acceptedParameters = ['w', 'w0', 'z', 'zr', 'g', 'r', 'q', 'Rc']
     
     def __init__(self, name, direction, parameter, node, alternate_beam=False):
         BaseDetector.__init__(self, name, node)
@@ -413,7 +508,7 @@ class bp(Detector1):
             raise pkex.BasePyKatException('Gouy detector code "{0}" is not a valid FINESSE command'.format(text))
             
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
 
         if self.alternate_beam:
             alt = "*"
@@ -476,14 +571,14 @@ class pd(Detector1):
         self.__f4 = Param("f4", self, None)
         self.__f5 = Param("f5", self, None)
         
-        self.__phi1 = Param("phi1", self, None)
-        self.__phi2 = Param("phi2", self, None)
-        self.__phi3 = Param("phi3", self, None)
-        self.__phi4 = Param("phi4", self, None)
-        self.__phi5 = Param("phi5", self, None)
+        self.__phase1 = Param("phase1", self, None)
+        self.__phase2 = Param("phase2", self, None)
+        self.__phase3 = Param("phase3", self, None)
+        self.__phase4 = Param("phase4", self, None)
+        self.__phase5 = Param("phase5", self, None)
         
         fs = [self.__f1, self.__f2, self.__f3, self.__f4, self.__f5]
-        ps = [self.__phi1, self.__phi2, self.__phi3, self.__phi4, self.__phi5]
+        ps = [self.__phase1, self.__phase2, self.__phase3, self.__phase4, self.__phase5]
 
         for i in range(num_demods):
             f = 'f{0}'.format(i+1)
@@ -494,15 +589,15 @@ class pd(Detector1):
             else:
                 raise pkex.BasePyKatException("Missing demodulation frequency {0} (f{0})".format(i+1))    
         
-            p = 'phi{0}'.format(i+1)
+            p = 'phase{0}'.format(i+1)
             
             if p in kwargs:
                 if kwargs[p] is None and i<num_demods-1:
-                    raise pkex.BasePyKatException("Missing demodulation phase {0} (phi{0})".format(i+1))
+                    raise pkex.BasePyKatException("Missing demodulation phase {0} (phase{0})".format(i+1))
                     
                 ps[i].value = kwargs[p]
             elif i<num_demods-1:
-                raise pkex.BasePyKatException("Missing demodulation phase {0} (phi{0})".format(i+1))
+                raise pkex.BasePyKatException("Missing demodulation phase {0} (phase{0})".format(i+1))
         
    
         self.__set_demod_attrs()
@@ -552,7 +647,7 @@ class pd(Detector1):
         elif isinstance(value, six.string_types) and not isinstance(value,float) and value.lower() != "max":
             raise pkex.BasePyKatException("Demodulation phase can only be set to a 'max' or a number (or None if the last demodulation phase)")
           
-        p = getattr(self, '_pd__phi' + num)  
+        p = getattr(self, '_pd__phase' + num)  
         p.value = value
         
     def __set_demod_attrs(self):
@@ -575,14 +670,14 @@ class pd(Detector1):
                         
                         setattr(self.__class__, "f"+name, property(fget=fget, fset=fset))
                     
-                    if not hasattr(self, "phi"+name):
-                        setattr(self.__class__, "phi"+name, property(fget=lambda self, i=i: self.__get_fphi('phi'+str(i)), fset=lambda self, value, i=i: self.__set_phi(str(i), value)))
+                    if not hasattr(self, "phase"+name):
+                        setattr(self.__class__, "phase"+name, property(fget=lambda self, i=i: self.__get_fphi('phase'+str(i)), fset=lambda self, value, i=i: self.__set_phi(str(i), value)))
                 else:
                     if hasattr(self, "f"+name):
                         delattr(self.__class__, "f"+name)
                         
-                    if hasattr(self, "phi"+name):
-                        delattr(self.__class__, "phi"+name)
+                    if hasattr(self, "phase"+name):
+                        delattr(self.__class__, "phase"+name)
         else:
             return
     
@@ -620,9 +715,9 @@ class pd(Detector1):
             dict['f{0}'.format(i+1)] = SIfloat(f[i])
         for i in range(len(phs)):
             if phs[i] == "max":
-                dict['phi{0}'.format(i+1)] = "max"
+                dict['phase{0}'.format(i+1)] = "max"
             else:
-                dict['phi{0}'.format(i+1)] = SIfloat(phs[i])
+                dict['phase{0}'.format(i+1)] = SIfloat(phs[i])
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -634,7 +729,7 @@ class pd(Detector1):
 
         
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.enabled:
             alt_str = ""
@@ -651,7 +746,7 @@ class pd(Detector1):
                 else:
                     fphi_str += " {0:.16g}".format(float(_f))
                     
-                phi_val = self.__getattribute__("phi"+str(n))
+                phi_val = self.__getattribute__("phase"+str(n))
                 
                 if phi_val != None:
                     if type(phi_val) == float:
@@ -728,7 +823,7 @@ class qnoised(pd):
         for i in range(len(f)):
             dict['f{0}'.format(i+1)] = f[i]
         for i in range(len(phs)):
-            dict['phi{0}'.format(i+1)] = phs[i]
+            dict['phase{0}'.format(i+1)] = phs[i]
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -746,7 +841,7 @@ class qnoised(pd):
         return qnoised(values[1], demods, node, senstype=sens, alternate_beam=alt_beam, **dict)
     
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.enabled:
             alt_str = ""
@@ -763,7 +858,7 @@ class qnoised(pd):
                 else:
                     fphi_str += " {0:.16g}".format(float(_f))
                     
-                phi_val = self.__getattribute__("phi"+str(n))
+                phi_val = self.__getattribute__("phase"+str(n))
                 
                 if phi_val != None:
                     if type(phi_val) == float:
@@ -826,7 +921,7 @@ class qshot(pd):
         for i in range(len(f)):
             dict['f{0}'.format(i+1)] = f[i]
         for i in range(len(phs)):
-            dict['phi{0}'.format(i+1)] = phs[i]
+            dict['phase{0}'.format(i+1)] = phs[i]
             
         node = values[-1]
         alt_beam = node[-1] == '*'
@@ -844,7 +939,7 @@ class qshot(pd):
         return qshot(values[1], demods, node, senstype=sens, alternate_beam=alt_beam, **dict)
     
     def getFinesseText(self) :
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.enabled:
             alt_str = ""
@@ -861,7 +956,7 @@ class qshot(pd):
                 else:
                     fphi_str += " {0:.16g}".format(float(_f))
                     
-                phi_val = self.__getattribute__("phi"+str(n))
+                phi_val = self.__getattribute__("phase"+str(n))
                 
                 if phi_val != None:
                     if type(phi_val) == float:
@@ -882,42 +977,6 @@ class qshot(pd):
                 rtn.extend(p.getFinesseText())
             
         return rtn
-        
-def xd(Detector1):
-
-    def __init__(self, name, node_name, component, motion):
-        BaseDetector.__init__(name, None)
-        
-        self.__motion = motion
-        self.__component = component
-        
-    @property
-    def motion(self): return self.__motion
-    
-    @property
-    def component(self): return self.__component
-    
-    @staticmethod
-    def parseFinesseText(text): 
-        values = text.split()
-
-        if len(values) != 4:
-            raise pkex.BasePyKatException("Motion detector command format incorrect '{0}' (2)".format(text))
-            
-        return xd(values[1], values[2], values[3])
-    
-    def getFinesseText(self) :
-        rtn = []
-        
-        if self.enabled:
-            rtn.append("xd {0} {1} {2}".format(self.name, self.component, self.motion))
-
-            self._getScaleCmds(rtn)
-                
-            for p in self._params:
-                rtn.extend(p.getFinesseText())
-            
-        return rtn
     
     
 class hd(Detector2):
@@ -942,7 +1001,7 @@ class hd(Detector2):
         return hd(values[1], float(values[2]), str(values[3]), str(values[4]))
     
     def getFinesseText(self):
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.enabled:   
             n1 = self.node1.name
@@ -1001,7 +1060,7 @@ class qhd(Detector2):
         return qhd(values[1], float(values[2]), str(values[3]), str(values[4]), sensitivity = sens)
     
     def getFinesseText(self):
-        rtn = []
+        rtn = BaseDetector.getFinesseText(self)
         
         if self.enabled:   
             n1 = self.node1.name
